@@ -1,16 +1,17 @@
-use mil::{
-    parser, executor,
-    cmdline::BuildCmd,
-    parser::ParseError,
-    executor::{ExecutionEnv, CovEnv},
-    compiler::{Compile, BinCode}};
-use std::fs::File;
-use std::path::PathBuf;
-use std::io::prelude::*;
-use structopt::StructOpt;
-use tmelcrypt::ed25519_keygen;
-use themelio_stf::Transaction;
 use anyhow::anyhow;
+use mil::{
+    cmdline::BuildCmd,
+    compiler::{BinCode, Compile},
+    executor,
+    executor::{CovEnv, ExecutionEnv},
+    parser,
+    parser::ParseError,
+};
+use std::fs::File;
+use std::io::prelude::*;
+use std::path::PathBuf;
+use structopt::StructOpt;
+use themelio_stf::Transaction;
 
 /// List of transactions and coin inputs to execute a script on.
 type TestTxs = Vec<(CovEnv, Transaction)>;
@@ -22,49 +23,13 @@ fn read_txs(fp: PathBuf) -> anyhow::Result<TestTxs> {
     file.read_to_string(&mut str_txs)?;
 
     // TODO: Don't expect here
-    Ok(serde_json::from_str(&str_txs)
-        .expect("Failed to parse transactions as json."))
+    Ok(serde_json::from_str(&str_txs).expect("Failed to parse transactions as json."))
 }
 
-/*
-fn execute_on_txs(txs: TestTxs, bin: &BinCode) {
-    let cov_hash = &tmelcrypt::hash_single(&bincode.0);
-    // Disassemble compiled binary
-    if let Some(ops) = executor::disassemble(bincode) {
-        //println!("Disassembly:\n{:?}\n", ops);
-
-        //if let Some(fp) = cmd.test_txs {
-            //let l = read_txs(fp)?;
-            let execs = txs.iter()
-                .map(|tx| executor::execute( executor::ExecutionEnv::new(&tx, &ops, cov_hash) ));
-
-            execs.for_each(|res| match res {
-                Some(final_state) => {
-                    println!("Successful execution.\n");
-                    println!("Final stack\n--------\n{:?}", final_state.0);
-                },
-                None => {
-                    println!("Execution failed.");
-                },
-            });
-        }
-    }
-}
-*/
-/*
-fn parse(code: &str) -> Result<MelExpr, ()> {
-    let mel_ops = parser::parse(&code[..])
-        .map_err(|e| match e {
-            ParseError::Syntax(e) => match e {
-                nom::Err::Failure(e) | nom::Err::Error(e) => format!("{}", nom::error::convert_error(&code[..], e)),
-                _ => unreachable!(),
-            },
-            ParseError::Expansion(msg) => format!("{}", msg.0),
-        })?;
-*/
-
-//fn main() -> std::io::Result<()> {
 fn main() -> anyhow::Result<()> {
+    env_logger::Builder::from_env("RUST_LOG")
+        .parse_filters("mil=debug,warn")
+        .init();
     // Command line arguments
     let cmd: BuildCmd = StructOpt::from_args();
 
@@ -73,15 +38,15 @@ fn main() -> anyhow::Result<()> {
     file.read_to_string(&mut code)?;
 
     // Parse to MelExpr ops
-    let mel_ops = parser::parse(&code[..])
-        .map_err(|e| match e {
-            ParseError::Syntax(e) => match e {
-                nom::Err::Failure(e) | nom::Err::Error(e) =>
-                    anyhow!(format!("{}", nom::error::convert_error(&code[..], e))),
-                _ => unreachable!(),
-            },
-            ParseError::Expansion(msg) => anyhow!(format!("{}", msg.0)),
-        })?;
+    let mel_ops = parser::parse(&code[..]).map_err(|e| match e {
+        ParseError::Syntax(e) => match e {
+            nom::Err::Failure(e) | nom::Err::Error(e) => {
+                anyhow!(nom::error::convert_error(&code[..], e))
+            }
+            _ => unreachable!(),
+        },
+        ParseError::Expansion(msg) => anyhow!(msg.0),
+    })?;
 
     // Compile to binary
     let empty = BinCode(Vec::new());
@@ -99,8 +64,7 @@ fn main() -> anyhow::Result<()> {
     println!("{}", address);
 
     // Disassemble compiled binary
-    let ops = executor::disassemble(bincode)
-        .ok_or("Failed to disassemble binary.").unwrap();
+    let ops = executor::disassemble(bincode).expect("Failed to disassemble binary.");
 
     // Show disassembly of binary if asked to
     if cmd.show_disassembly {
@@ -117,38 +81,45 @@ fn main() -> anyhow::Result<()> {
                 println!("Debug execution log for tx#{}", i);
                 //println!("{:?}", serde_json::to_string(&tx));
 
-                let mut env = ExecutionEnv::new(tx, cov_env, &ops);
+                let env = ExecutionEnv::new(tx, cov_env, ops.clone());
                 // Display every step in debug mode
-                env.into_iter()
+                env.iterate()
                     .take_while(|r| r.is_some())
                     .inspect(|res| match res {
-                        Some((stack,heap,pc)) =>
-                            println!("-----\n\
+                        Some((stack, heap, pc)) => println!(
+                            "-----\n\
                                 Executed instruction: {:?}\n\
                                 Next instruction: {:?}\n\n\
                                     Stack\n{:?}\n\n\
                                     Heap\n{:?}\n",
-                                ops[*pc-1], ops.get(*pc), stack, heap),
+                            ops[*pc - 1],
+                            ops.get(*pc),
+                            stack,
+                            heap
+                        ),
                         None => (),
                     })
                     .last();
             });
         } else {
-            let execs = l.into_iter()
-                .map(|(cov_env, tx)|
-                    executor::execute(
-                        ExecutionEnv::new(tx, cov_env, &ops)));
+            let weights: Vec<u128> = l.iter().map(|(_, tx)| tx.weight()).collect();
+            let execs = l.into_iter().map(|(cov_env, tx)| {
+                executor::execute(ExecutionEnv::new(tx, cov_env, ops.clone()))
+            });
+
+            execs.enumerate().for_each(|(i, res)| {
+                // Show weight of the transaction
+                println!("Transaction weight: {}", weights[i]);
 
-            execs.enumerate().for_each(|(i,res)| {
                 print!("tx#{} - ", i);
                 match res {
                     Some(final_state) => {
                         println!("Successful execution.\n");
                         println!("Final stack\n--------\n{:?}", final_state.0);
-                    },
+                    }
                     None => {
                         println!("Execution failed.");
-                    },
+                    }
                 }
             });
         }
